#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_onionrouter
----------------------------------

Tests for `onionrouter` module.
"""

import pytest
from onionrouter.routers import OnionPostfixRerouter


class TestOnionRouter(object):
    def test_multiple_hostnames_support(self, dummy_onionrouter):
        assert len(dummy_onionrouter.myname) > 1

    def test_hostname_is_upper(self, dummy_onionrouter):
        assert all(map(lambda x: x.isupper(), dummy_onionrouter.myname))

    def test_get_domain_multiple_at(self, dummy_onionrouter):
        with pytest.raises(RuntimeError):
            dummy_onionrouter.get_domain("lalla@lalala.com@lalal.net")

    def test_get_domain_no_email(self, dummy_onionrouter):
        with pytest.raises(RuntimeError):
            dummy_onionrouter.get_domain("lalla")

    def test_get_domain_correct_email(self, dummy_onionrouter):
        assert dummy_onionrouter.get_domain("lol@test.com") == "test.com"

    def test_reroute_local_domain(self, dummy_onionrouter):
        assert dummy_onionrouter.reroute("myself.net") == tuple(["200 :"])

    def test_reroute_no_lazy_config(self, dummy_onionrouter,
                                    monkeypatch):
        dummy_answer = tuple(["test"])
        monkeypatch.setattr(OnionPostfixRerouter, "reroute",
                            lambda *args: dummy_answer)
        assert dummy_onionrouter.reroute("testme") == dummy_answer

    def test_reroute_run_wrong_domain(self, dummy_onionrouter):
        assert (dummy_onionrouter.run("llalalla")
                == "500 Request key is not an email address")

    def test_reroute_multiple_findings(self, dummy_onionrouter,
                                       monkeypatch):
        dummy_answer = tuple(["test1", "test2", "test3"])
        monkeypatch.setattr(OnionPostfixRerouter, "reroute",
                            lambda *args: dummy_answer)
        assert dummy_onionrouter.run("t@t.c") == dummy_answer[0]

    def test_ignored_domains_exist(self, dummy_onionrouter):
        assert dummy_onionrouter.ignored_domains

    def test_ignored_domains_case_insensitive(self, dummy_onionrouter):
        assert all(map(lambda x: x.isupper(),
                       dummy_onionrouter.ignored_domains))

    def test_ignored_domain_rerouting(self, dummy_onionrouter):
        assert (dummy_onionrouter.run("please@iGnORe.Me")
                == "500 Domain is in ignore list")

    def test_check_local_domain_before_ignored(self, dummy_onionrouter):
        assert dummy_onionrouter.run("m@myself.net") == "200 :"



import sys
import os
import re


def cross_input(text):
    """
    Returns the correct input function callback to be used for python 3.x
    and python 2.x
    """
    if sys.version_info[0] < 3:
        return raw_input(text)  # noqa
    return input(text)


def get_full_path_files_for_dir(directory_path):
    """
    List all filepaths in a dir recursively
    """
    return set([os.path.join(dirpath, fname) for dirpath, dirnames, filenames
                in tuple(os.walk(directory_path))
                for fname in filenames])


def find_file(directory_path, filename):
    """
    Find filename in a dir recursively
    """
    return next((x for x in get_full_path_files_for_dir(directory_path)
                 if filename == os.path.split(x)[-1]), None)


def find_files_with_suffix(directory_path, suffix):
    all_files = get_full_path_files_for_dir(directory_path)
    matcher = re.compile(r".*\.{suffix}$".format(suffix=suffix), re.IGNORECASE)
    return (a_file for a_file in all_files if re.match(matcher, a_file))



from __future__ import unicode_literals
import pytest
import io
try:
    import configparser
except ImportError:
    import ConfigParser as configparser
from onionrouter import rerouter, config_handlers


config = """
[RESOLVER]
resolver_ip: 127.0.0.1
resolver_port: 53
tcp: True

[DOMAIN]
hostname: myself.net, myself2.net

[DNS]
srv_record: _onion-mx._tcp.

[REROUTE]
onion_transport: smtptor

[IGNORED]
domains: ignore.me, ignore2.me
"""


@pytest.fixture(scope="session", name="dummy_config")
def fixture_config():
    return config


@pytest.fixture(scope="function", name="dummy_onionrouter")
def fixture_onionrouter(monkeypatch, dummy_config):
    monkeypatch.setattr(
        config_handlers, "get_conffile",
        lambda *args, **kwargs: rerouter.OnionRouter.ref_config)
    custom_config = configparser.ConfigParser()
    custom_config._read(io.StringIO(dummy_config), None)
    monkeypatch.setattr(config_handlers, "config_reader",
                        lambda *args: custom_config)
    return rerouter.OnionRouter("nothing?")




#!/usr/bin/env python
from __future__ import print_function
import argparse
from collections import namedtuple
from functools import partial
from pkg_resources import resource_filename
import sys
from socket import error as socket_error
from onionrouter.lookups import OnionServiceLookup
from onionrouter import (config_handlers, custom_exceptions as exc,
                         msockets, olib, routers)


default_config_path = "/etc/onionrouter/"
default_mappings_path = "/etc/onionrouter/mappings"


class OnionRouter(object):
    ref_config = (olib.find_file("/etc/onionrouter/", "onionrouter.ini")
                  or resource_filename("onionrouter",
                                       "configs/onionrouter.ini"))
    rerouters = namedtuple('rerouters', ('lazy', 'onion'))

    def __init__(self, config_path, map_path=""):
        self.config_file = config_handlers.get_conffile(config_path,
                                                        prefix="onionrouter")
        self.mappings_path = map_path
        config_handlers.ConfigIntegrityChecker(
            ref_config=self.ref_config, other_config=self.config_file).verify()
        self.config = config_handlers.config_reader(self.config_file)
        self.rerouters = self.rerouters(
            lazy=routers.LazyPostfixRerouter(self.config, self.mappings_path),
            onion=routers.OnionPostfixRerouter(
                self.config, OnionServiceLookup(self.config)))

    @property
    def myname(self):
        return [x.strip().upper() for x in self.config.get(
            "DOMAIN", "hostname").split(",")]

    @property
    def ignored_domains(self):
        return [x.strip().upper() for x in self.config.get(
            "IGNORED", "domains").split(",")]

    @staticmethod
    def get_domain(address):
        split_addr = address.split("@")
        if address.count("@") != 1 or split_addr[1] == "":
            raise RuntimeError
        return split_addr[1]

    def reroute(self, domain):
        if domain.upper() in self.myname:
            return tuple(["200 :"])
        elif domain.upper() in self.ignored_domains:
            return tuple(["500 Domain is in ignore list"])
        else:
            return (self.rerouters.lazy.reroute(domain)
                    or self.rerouters.onion.reroute(domain))

    def run(self, address):
        try:
            domain = self.get_domain(address)
        except RuntimeError:
            return "500 Request key is not an email address"
        routing = self.reroute(domain)
        # in the end, there can be only one response
        return routing[0] if routing else "500 Not found"


class OnionRouterRunner(object):

    @staticmethod
    def add_arguments():
        parser = argparse.ArgumentParser(
            description='onionrouter daemon for postifx rerouting')
        parser.add_argument('--interactive', '-i', default=False,
                            action='store_true',
                            help='Simple test route mode without daemon')
        parser.add_argument('--debug', '-d', default=False,
                            action='store_true',
                            help='Debug mode. Run daemon and also print the '
                                 'queries & replies')
        parser.add_argument('--client', '-c', default=False,
                            action='store_true',
                            help='Client mode. Connect as a client to daemon '
                                 'for testing / debug')
        parser.add_argument('--config', default=default_config_path,
                            help='Absolute path to config folder/file '
                                 '(default: %(default)s)', type=str)
        parser.add_argument('--mappings', default=default_mappings_path,
                            help='Absolute path to static mappings folder/file'
                                 ' (default: %(default)s)', type=str)
        parser.add_argument('--host', '-l', default="127.0.0.1",
                            help="Host for daemon to listen "
                                 "(default: %(default)s)", type=str)
        parser.add_argument('--port', '-p', default=23000, type=int,
                            help="Port for daemon to listen "
                                 "(default: %(default)s)")
        return parser

    @staticmethod
    def interactive_reroute(onion_router):
        while True:
            addr = olib.cross_input("Enter an email address: ")
            if addr == 'get *':
                print("500 Request key is not an email address")
            else:
                print(onion_router.run(addr))

    @staticmethod
    def reroute_debugger(question, answer):
        print("[Q]: {q}\n"
              "[A]: {a}".format(q=question, a=answer))

    @staticmethod
    def craft_resolver(callback):
        return partial(msockets.resolve, resolve_callback=callback)

    @staticmethod
    def validate_flag_arguments(*flags):
        if len([x for x in flags if x]) > 1:
            raise RuntimeWarning("Cannot use multiple modes. "
                                 "Choose only one mode")

    def main(self):
        args = self.add_arguments().parse_args()
        self.validate_flag_arguments(args.interactive, args.client, args.debug)
        onion_router = OnionRouter(config_path=args.config,
                                   map_path=args.mappings)

        if args.interactive:
            self.interactive_reroute(onion_router)

        if args.client:
            msockets.client(args.host, args.port)
        else:
            resolver = self.craft_resolver(self.reroute_debugger
                                           if args.debug
                                           else lambda *args: args)
            msockets.daemonize_server(onion_router, args.host, args.port,
                                      resolver=resolver)


def main():
    try:
        OnionRouterRunner().main()
    except (exc.ConfigError, socket_error, RuntimeWarning) as err:
        print(err)
        sys.exit(1)
    except KeyboardInterrupt:
        sys.exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "dnspython<=2.0.0",
    "PyYAML==4.2b1",
]

test_requirements = [
    "pip==8.1.2",
    "bumpversion==0.5.3",
    "wheel==0.29.0",
    "watchdog==0.8.3",
    "flake8==2.6.0",
    "tox==2.3.1",
    "coverage==4.1",
    "Sphinx==1.4.8",
    "cryptography==1.7",
    "PyYAML==4.2b1",
    "pytest==2.9.2"
]

setup(
    name='onionrouter',
    version='0.5.4',
    description="Python Onion Routed Mail Deliveries",
    long_description=readme,
    author="Ehlo Onion",
    author_email='onionmx@lists.immerda.ch',
    url='https://github.com/ehloonion/onionrouter',
    packages=[
        'onionrouter',
    ],
    entry_points={
        "console_scripts": ['onionrouter = onionrouter.rerouter:main']
    },
    include_package_data=True,
    install_requires=requirements,
    license="GNU General Public License v3",
    zip_safe=False,
    keywords='onionrouter',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



from abc import abstractmethod, ABCMeta
from dns import exception as dnsexception
from onionrouter.config_handlers import load_yamls


class PostfixRerouter(object):
    __metaclass__ = ABCMeta

    def __init__(self, config):
        self.config = config

    @property
    def reroute_service(self):
        return self.config.get("REROUTE", "onion_transport")

    @abstractmethod
    def reroute(self, answers):
        return tuple("200 {rservice}:[{ans}]".format(
            rservice=self.reroute_service, ans=x) for x in answers)


class LazyPostfixRerouter(PostfixRerouter):
    def __init__(self, config, yaml_path):
        super(LazyPostfixRerouter, self).__init__(config)
        self.mappings_path = yaml_path
        self.static_mappings = dict()

    def _setup_static_mappings(self):
        for (key, vals) in load_yamls(self.mappings_path).items():
            self.static_mappings.update({x.upper(): key for x in vals})

    def _lazy(self, domain):
        mapping = self.static_mappings.get(domain.upper())
        return tuple([mapping]) if mapping else tuple()

    def reroute(self, domain):
        if not self.static_mappings:
            self._setup_static_mappings()
        return super(LazyPostfixRerouter, self).reroute(self._lazy(domain))


class OnionPostfixRerouter(PostfixRerouter):
    def __init__(self, config, onion_resolver):
        super(OnionPostfixRerouter, self).__init__(config)
        self.onion_resolver = onion_resolver

    def reroute(self, domain):
        try:
            return super(OnionPostfixRerouter, self).reroute(
                self.onion_resolver.lookup(domain))
        except dnsexception.DNSException as err:
            return tuple(("500 {0}".format(str(err) or "Not found"),))
